import { InjectionToken, Inject, Injectable, NgModule, Optional, SkipSelf } from '@angular/core';
import { Title, DOCUMENT } from '@angular/platform-browser';
import { Router, NavigationEnd, ActivatedRoute, RouterModule } from '@angular/router';
import 'rxjs/add/operator/filter';
import 'rxjs/add/operator/map';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Additional site-wide configuration of meta tags.
 *
 * Use this to enable adding a title suffix to the title of each route,
 * or to set default meta tags to be used when no equivalent meta tag
 * exists in the meta configuration of the route.
 *
 *
 * Default
 * ```
 * {
 *   useTitleSuffix: false,
 *   defaults: {}
 * }
 * ```
 * @record
 */

const META_CONFIG_TOKEN = new InjectionToken('meta config');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const isDefined = (val) => typeof val !== 'undefined';
class MetaService {
    /**
     * @param {?} router
     * @param {?} document
     * @param {?} titleService
     * @param {?} activatedRoute
     * @param {?} metaConfig
     */
    constructor(router, document, titleService, activatedRoute, metaConfig) {
        this.router = router;
        this.document = document;
        this.titleService = titleService;
        this.activatedRoute = activatedRoute;
        this.metaConfig = metaConfig;
        this.router.events
            .filter((event) => (event instanceof NavigationEnd))
            .map(() => this._findLastChild(this.activatedRoute))
            .subscribe((routeData) => {
            this._updateMetaTags(routeData.meta);
        });
    }
    /**
     * @param {?} activatedRoute
     * @return {?}
     */
    _findLastChild(activatedRoute) {
        const /** @type {?} */ snapshot = activatedRoute.snapshot;
        let /** @type {?} */ child = snapshot.firstChild;
        while (child.firstChild !== null) {
            child = child.firstChild;
        }
        return child.data;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    _getOrCreateMetaTag(name) {
        let /** @type {?} */ el = this.document.querySelector(`meta[name='${name}']`);
        if (!el) {
            el = this.document.createElement('meta');
            el.setAttribute('name', name);
            this.document.head.appendChild(el);
        }
        return el;
    }
    /**
     * @param {?=} meta
     * @return {?}
     */
    _updateMetaTags(meta = {}) {
        if (meta.disableUpdate) {
            return false;
        }
        this.setTitle(meta.title, meta.titleSuffix);
        Object.keys(meta).forEach(key => {
            if (key === 'title' || key === 'titleSuffix') {
                return;
            }
            this.setTag(key, meta[key]);
        });
        Object.keys(this.metaConfig.defaults).forEach(key => {
            if (key in meta || key === 'title' || key === 'titleSuffix') {
                return;
            }
            this.setTag(key, this.metaConfig.defaults[key]);
        });
    }
    /**
     * @param {?=} title
     * @param {?=} titleSuffix
     * @return {?}
     */
    setTitle(title, titleSuffix) {
        const /** @type {?} */ titleElement = this._getOrCreateMetaTag('title');
        const /** @type {?} */ ogTitleElement = this._getOrCreateMetaTag('og:title');
        let /** @type {?} */ titleStr = isDefined(title) ? title : (this.metaConfig.defaults['title'] || '');
        if (this.metaConfig.useTitleSuffix) {
            titleStr += isDefined(titleSuffix) ? titleSuffix : (this.metaConfig.defaults['titleSuffix'] || '');
        }
        titleElement.setAttribute('content', titleStr);
        ogTitleElement.setAttribute('content', titleStr);
        this.titleService.setTitle(titleStr);
        return this;
    }
    /**
     * @param {?} tag
     * @param {?} value
     * @return {?}
     */
    setTag(tag, value) {
        if (tag === 'title' || tag === 'titleSuffix') {
            throw new Error(`Attempt to set ${tag} through 'setTag': 'title' and 'titleSuffix' are reserved tag names.
      Please use 'MetaService.setTitle' instead`);
        }
        const /** @type {?} */ tagElement = this._getOrCreateMetaTag(tag);
        let /** @type {?} */ tagStr = isDefined(value) ? value : (this.metaConfig.defaults[tag] || '');
        tagElement.setAttribute('content', tagStr);
        if (tag === 'description') {
            let /** @type {?} */ ogDescElement = this._getOrCreateMetaTag('og:description');
            ogDescElement.setAttribute('content', tagStr);
        }
        return this;
    }
}
MetaService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
MetaService.ctorParameters = () => [
    { type: Router, },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
    { type: Title, },
    { type: ActivatedRoute, },
    { type: undefined, decorators: [{ type: Inject, args: [META_CONFIG_TOKEN,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MetaModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('MetaModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * @param {?=} metaConfig
     * @return {?}
     */
    static forRoot(metaConfig = { useTitleSuffix: false, defaults: {} }) {
        return {
            ngModule: MetaModule,
            providers: [
                { provide: META_CONFIG_TOKEN, useValue: metaConfig },
                MetaService
            ]
        };
    }
}
MetaModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    RouterModule
                ]
            },] },
];
/** @nocollapse */
MetaModule.ctorParameters = () => [
    { type: MetaModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { MetaService, MetaModule, META_CONFIG_TOKEN as ɵa };
//# sourceMappingURL=ng2-meta.js.map
